###############################################################################################################
# Module: script.py
# Author: Romain Lienard (rlienard@gmail.com)
# Description : A library providing several wrappers for IBM Security Secret Server REST APIs
# License : Apache Licence version 2.0 + Cookieware variant - if you like this module, please send me biscuits
###############################################################################################################

class Script:
    
    extensions = {1:"ps",2:"sql",3:"sh"}
    
    def __init__(self,url,payload,token,scriptList=False):
        self._url = url
        self._payload = payload
        self._token = token
        self._scriptList = scriptList
        
        if not scriptList:
            self._id = payload["d"]["ScriptId"]
            self.scriptType = payload["d"]["ScriptType"]
            self.name = payload["d"]["Name"]
            self.content = payload["d"]["Script"]
            self.description = payload["d"]["Description"]
            self.active = payload["d"]["Active"]
            self.scriptCategoryId = payload["d"]["ScriptCategoryId"]    
        else:
            self._id = payload["ScriptId"]
            self.scriptType = payload["ScriptType"]
            self.name = payload["Name"]
            self.content = payload["Script"]
            self.description = payload["Description"]
            self.active = payload["Active"]
            self.scriptCategoryId = payload["ScriptCategoryId"] 
            
        self.filename = self.name + "." + Script.extensions[self.scriptType]
        
    def __str__(self):
        return '<ISSS_Script id="%s" name="%s" description="%s" content="%s" />' %(self._id,self.name,self.description,self.content)
    
    def update(self):
        if self._scriptList:
            raise Exception("Cannot update this script. You need to retrieve the full content using isss.getScript(<script>) first")
            
        updatePayload = {"script":
                {
                 "ScriptId":self._id,
                 "Name":self.name,
                 "Description":self.description,
                 "ScriptCategoryId":self.scriptCategoryId,
                 "Script":self.content,
                 "Active":self.active,
                 "ScriptType":self.scriptType,
                 "AdditionalData":self._payload["d"]["AdditionalData"]
                 }
        }
        
        updatePayload = json.dumps(updatePayload)
        
        try:
            req = urllib.request.Request(
                self._url + "/ajax/AjaxServices.asmx/UpdateScript",
                updatePayload.encode("utf-8"),
                headers={
                    "User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
                    "Content-Type":"application/json; charset=UTF-8",
                    "X-Requestverificationtoken":self._token,
                    "Accept" : "application/json, text/javascript, */*; q=0.01",
                    "X-Requested-With" : "XMLHttpRequest"
                }
            )
            parsed = json.load(opener.open(req))
            
            return parsed["d"]["Success"]
        
        except Exception as e:
            print (e)
            return False


#!/usr/bin/env python3

##############################################################################################################
# Module: isss.py
# Author: Romain Lienard (rlienard@gmail.com)
# Description : A library providing several wrappers for IBM Security Secret Server REST APIs
# License : Apache Licence version 2.0 + Cookieware variant - if you like this module, please send me biscuits
###############################################################################################################

import json
import urllib.request, urllib.parse, urllib.error,urllib.request,urllib.error,urllib.parse
import ssl
from http.cookiejar import CookieJar
from hashlib import md5
import re
import shutil
import os,os.path
import time
import tkinter
import sys
from threading import Thread
import sys
import time

cj = CookieJar()
ignoreSSL = ssl._create_unverified_context()
handler = urllib.request.HTTPSHandler(context=ignoreSSL)
opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(cj),handler)

version = "1.0.2"
    

from isss.stubs import *
from isss.script import Script

    
USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36"    

class ISSS:
    
    def __init__(self,url,username,password,experimentalMode=False,ignoreWarning=False):
        self.OWNER = 1
        self.EDIT = 2
        self.VIEW = 3
        self.LIST = 4
        self.loginParams = {}
        self.url = url
        self.username = username
        self.password = password
        self.api = "/api/v1"
        self.cacheToken = None
        self.expireToken = None
        
        if experimentalMode:
            if not ignoreWarning:
                print("experimental mode is enabled. things might not work as expected :-)")
            self.login(username,password)
            self.adminScriptsToken = self.getTokenFromUrl("/AdminScripts.aspx","window.aftToken") 
    
    
    def getAccessToken(self):
        if not self.cacheToken or (self.expireToken < time.time()):
            data = {"username":self.username,"password":self.password,"grant_type":"password"}
            req = urllib.request.Request(self.url + "/oauth2/token",urllib.parse.urlencode(data).encode("utf-8"))
            res = urllib.request.urlopen(req,context=ignoreSSL)
            self.cacheToken = json.load(res)["access_token"]
            self.expireToken = time.time()+300
        
        return self.cacheToken
    
    def get(self,url,params=None):
        url = self.url + self.api + url
        
        if params and params != "":
            url+="?" + urllib.parse.urlencode(params)
        try:
            req = urllib.request.Request(url,headers={"Authorization":"Bearer " + self.getAccessToken()})
            return json.load(urllib.request.urlopen(req,context=ignoreSSL))
        
        except urllib.error.HTTPError as e:
            raise Exception("The server returned an error:\n" + e.read().decode())
        
        except Exception as e:
            print (e)
            raise Exception("Unexpected error.")
        
    def post(self,url,data="",method="POST"):
        url = self.url + self.api + url
        
        if data != "":
            data = json.dumps(data)
            
        try:
            req = urllib.request.Request(url,data.encode("utf-8"),headers={"Authorization":"Bearer " + self.getAccessToken(),"Content-Type":"application/json; charset=UTF-8",},method=method)
            return json.load(urllib.request.urlopen(req,context=ignoreSSL))
        
        except urllib.error.HTTPError as e:
            raise Exception("The server returned an error:\n" + e.read().decode())
            
        except Exception as e:
            print(e)
            raise Exception("Unexpected error.")
            
    def put(self,url,data=""):
        return self.post(url,data,"PUT")
    
    def delete(self,url,data=""):
        return self.post(url,data,"DELETE")
    
    def getAnything(self,url,idOrText=None):
        
        if isinstance(idOrText,int):
            return self.get(url+"/"+str(idOrText))
        
        elif isinstance(idOrText,str):
            params = {
                    "filter.searchText" : idOrText,
                    "take" : 100
                     }
            resp = self.get(url,params)
        else:
            resp = self.get(url)
            
        if resp != []:
            if "records" in resp:
                return resp["records"]
            else:
                return resp
        else:
            return []
        
    
    
    
    
    def getUser(self,idOrText=""):
        res = self.getAnything("/users",idOrText)
        if isinstance(idOrText,str):
            tab = []
            for r in res:
                tab.append(UserStub(self,r,False))
            return tab
        else:
            return UserStub(self,res,False)
    
    def getTemplate(self,idOrText=""):
        return self.getAnything("/secret-templates",idOrText)
    
    def getGroup(self,idOrText=""):
        res = self.getAnything("/groups",idOrText)
        if isinstance(idOrText,str):
            tab = []
            for r in res:
                tab.append(GroupStub(self,r,False))
            return tab
        else:
            return GroupStub(self,res,False)
    
    def getFolder(self,idOrText=""):
        res = self.getAnything("/folders",idOrText)  
        if isinstance(idOrText,str):
            tab = []
            for r in res:
                tab.append(FolderStub(self,r,False))
            return tab
        else:
            return FolderStub(self,res,False) 
    
    def getSecret(self,idOrText=""):
        res = self.getAnything("/secrets",idOrText) 
        if isinstance(idOrText,str):
            tab = []
            for r in res:
                tab.append(SecretStub(self,r,False))
            return tab
        else:
            return SecretStub(self,res,False) 

    def getSecretPermission(self,secretId):
        res = self.getAnything("/secret-permissions?filter.secretId=" + str(secretId)) 
        tab = []
        for r in res:
            tab.append(SecretPermissionStub(self,r,False))
        return tab
    
    def getFolderPermission(self,folderId):
        res = self.getAnything("/folder-permissions?filter.folderId=" + str(folderId)) 
        tab = []
        for r in res:
            tab.append(FolderPermissionStub(self,r,False))
        return tab
    
    
    
    
    
    
    def getUserStub(self):
        return UserStub(self,self.getAnything("/users/stub"))
    
    def getSecretStub(self,templateId):
        res = self.getAnything("/secrets",idOrText) 
        if isinstance(idOrText,str):
            tab = []
            for r in res:
                tab.append(SecretStub(self,r,False))
            return tab
        else:
            return SecretStub(self,res,False) 
    
    def getGroupStub(self):
        return GroupStub(self,self.getAnything("/groups/stub"))
    
    def getFolderStub(self):
        return FolderStub(self,self.getAnything("/folders/stub"))
    
    def getSecretPermissionStub(self,secretId):
        return SecretPermissionStub(self,self.getAnything("/secret-permissions/stub?secretId="+str(secretId)))
    
    def getFolderPermissionStub(self,folderId):
        return FolderPermissionStub(self,self.getAnything("/folder-permissions/stub?folderId="+str(folderId)))
    
    def createSecret(self,stub):    
        rep = self.post("/secrets",stub.payload)
        return rep
    def createUser(self,stub):    
        rep = self.post("/users",stub.payload)
        return rep
    
    def createGroup(self,stub):    
        rep = self.post("/groups",stub.payload)
        return rep
    
    def createSecretPermission(self,stub):    
        rep = self.post("/secret-permissions",stub.payload)
        return rep
    
    def createFolder(self,stub):    
        rep = self.post("/folders",stub.payload)
        return rep
    
    def createFolderPermission(self,stub):    
        rep = self.post("/folder-permissions",stub.payload)
        return rep
    
    
    
    def putAnything(self,url,payload):
        return self.put(url,payload)
    
    def putSecret(self,stub):
        rep = self.put("/secrets/"+str(stub["id"])+"/restricted",stub.payload)
        return rep
    
    def putUser(self,stub):
        rep = self.put("/users/"+str(stub["id"]),stub.payload)
        return rep
    
    def putSecretPermission(self,stub):
        rep = self.put("/secret-permissions/"+str(stub["id"]),stub.payload)
        return rep
    
    def putFolderPermission(self,stub):
        rep = self.put("/folder-permissions/"+str(stub["id"]),stub.payload)
        return rep
    
    def putGroup(self,stub):
        rep = self.put("/groups/"+str(stub["id"]),stub.payload)
        return rep
    
    def putFolder(self,stub):
        rep = self.put("/folders/"+str(stub["id"]),stub.payload)
        return rep
    
    
    def deleteSecret(self,stub):
        self.delete("/secrets/"+str(stub["id"]))
        
    def deleteUser(self,stub):
        self.delete("/users/"+str(stub["id"]))
        
    def deleteSecretPermission(self,stub):
        self.delete("/secret-permissions/"+str(stub["id"]))
        
    def deleteFolderPermission(self,stub):
        self.delete("/folder-permissions/"+str(stub["id"]))
        
    def deleteGroup(self,stub):
        self.delete("/groups/"+str(stub["id"]))
        
    def deleteFolder(self,stub):
        self.delete("/folders/"+str(stub["id"]))
    
    
    ####################################
    #Experimental section
    ####################################
    
    
    def getLoginPage(self):
        req = urllib.request.Request(self.url + "/login.aspx",headers={
            "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3"})
        #data = urllib2.urlopen(req,context=ignoreSSL).read()
        data = opener.open(req).read().decode()
        toSearch = ["__VIEWSTATE","__VIEWSTATEGENERATOR","__EVENTVALIDATION","__RequestVerificationToken"]

        for item in toSearch:
            r = re.compile('name="('+item+')" (id=".+?" )?(type="hidden" )?value="(.+?)"')
            for m in re.finditer(r,data):
                #print m.group(1),"=>",m.group(4)
                self.loginParams[m.group(1)] = m.group(4)
            
    def login(self,username,password):
        self.getLoginPage()
        url = self.url + "/login.aspx"
        data = {
                "LoginUserControl1$HasAddSecretPermissionControl":"false",
                "LoginUserControl1$UserNameTextBox":username,
                "LoginUserControl1$UserNameOriginalHiddenField":"",
                "LoginUserControl1$PasswordTextBox":password,
                "LoginUserControl1$LoginButton":"Login",
                "LoginUserControl1$LoginDialog_IsCollapsed":"0",
                "__SCROLLPOSITIONX" : "0",
                "__SCROLLPOSITIONY" : "0",
                "__EVENTTARGET" : "",
                "__EVENTARGUMENT" : ""
               }
        
        for k in self.loginParams:
            data[k] = self.loginParams[k]
        
        req = urllib.request.Request(url,urllib.parse.urlencode(data).encode("utf-8"),headers={
            "User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
            "Content-Type":"application/x-www-form-urlencoded",
            
        })
        data = opener.open(req).read()
        
   
    
    def getTokenFromUrl(self,url,name):
        req = urllib.request.Request(
        self.url + url,
        headers={
            "User-Agent":USER_AGENT,
            "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3"
        }
        )
        data = opener.open(req).read().decode()
        
        r = re.compile(name + " = '(.+?)'");
        for m in re.finditer(r,data):
            return m.group(1)
        
        return None
    
    
    def getScriptList(self):
        req = urllib.request.Request(
        self.url + "/AdminScripts.aspx",
        headers={
            "User-Agent":USER_AGENT,
            "Accept":"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3"
        }
        )
        data = opener.open(req).read().decode()
        
        r = re.compile("var scripts = (\\[.+?\\]);");
        
        for m in re.finditer(r,data):
            scripts = json.loads(m.group(1))
            result = []
            for s in scripts:
                script = Script(self.url,s,self.adminScriptsToken,True)
                result.append(script)
            return result
        else:
            return []
        

    def getScript(self, scriptOrScriptId):
        if isinstance(scriptOrScriptId,Script):
            scriptId = scriptOrScriptId._id
        else:
            scriptId = scriptOrScriptId
        
        try:
            req = urllib.request.Request(
                self.url + "/ajax/AjaxServices.asmx/GetPowershellScript",
                str({"powershellScriptId":scriptId}).encode("utf-8"),
                headers={
                    "User-Agent":USER_AGENT,
                    "Content-Type":"application/json; charset=UTF-8",
                    "X-Requestverificationtoken":self.adminScriptsToken
                }
            )
            parsed = json.load(opener.open(req))
            script = Script(self.url,parsed,self.adminScriptsToken)
            return script
        
        except Exception as e:
            print (e)
            return []

            
    
    def showPopup(self,message,color="black"):
        root = tkinter.Tk()
        
        def callback():
            root.destroy()
            
        root.title("python for isss")
        label = tkinter.Label(root, text=" " + message + " ",fg=color)
        btn = tkinter.Button(root, text="close",command=callback)
        label.pack()
        btn.pack()
        #root.after(2000, root.destroy)
        root.mainloop()
    
    def fw(self,directory):
        def getsum(directory,f):
            try:
                with open(os.path.join(directory,f)) as ff:
                    return md5(ff.read().encode("utf-8")).hexdigest()
            except Exception as e:
                print (e)
                print(("error while getsum('%s')"%(os.path.join(directory,f))))
        
        if os.path.isfile(directory):
            raise Exception ("The specified path is not a directory but an existing file.")
        
        if os.path.exists(directory):
            rep = input ("The specified directory already exists, existing content will be deleted. Continue (y/n) [y] ? ").strip().lower()
            if rep == "y" or rep == "":
                print(("Cleaning directory %s ..." % (directory)))
                shutil.rmtree(directory)
                os.mkdir(directory)
            else:
                print("Aborting.")
                return
        
        if not os.path.exists(directory):
            print(("Directory %s doesn't exist. Creating directory..." % (directory)))
            os.mkdir(directory)
        
        sys.stdout.write("Starting Filewatcher [----------]")
        
        scripts = self.getScriptList()
        scriptmap = {}
        
        if len(scripts) == 0:
            print ("\nUnable to retrieve any script. Please check your connection settings and try again.")
            sys.exit(1)
        
        sys.stdout.flush()
        
        pos = 0
        step = 100.0/len(scripts)
        
        for s in scripts:
            script = self.getScript(s)
            pos+=step
            prog = int(round(pos / 10.0,0))
            sys.stdout.write("\rStarting Filewatcher ["+ ("*" * prog) + ("-" * (10-prog)) + "]")
            sys.stdout.flush()
            scriptmap [s.filename] = script
            with open(os.path.join(directory,s.filename),"w") as out:
                out.write(script.content)
                
        sums = {}
        
        init = True
        
        sys.stdout.write( "\rFilewatcher is now open for e-business ! (press ctrl+c to quit)\n")
        sys.stdout.flush()

        try:
            while True:
                for f in os.listdir(directory):
                    if f not in sums:
                        if init:
                            sums[f] = getsum(directory,f)
                        else: 
                            print(("new file detected : " + f))
                            sums[f] = getsum(directory,f)
                    else:
                        newSum = getsum(directory,f)
                        if sums[f] != newSum:
                            print(("file modified : " + f))
                            sums[f] = newSum
                            if (re.match("^.+?[.](ps|sh|sql)$",f)):
                                with open(os.path.join(directory,f)) as ff:
                                    scriptmap[f].content = ff.read()
                                    if scriptmap[f].update(): #todo : find a way to display a popup which doesn't hang :-/
                                        print ("file updated successfully on server")
                                        #self.showPopup("Success : '%s' updated successfully on server"%(f),"green")
                                        
                                    else:
                                        print ("error while updating file on server")
                                        #self.showPopup("Error : '%s' has NOT been updated on server."%(f),"red")
                                        
                        else:
                            pass
                init = False
                time.sleep(1)
                
                
        except KeyboardInterrupt:
            print("Filewatcher is now stopped. Bye")
   



###############################################################################################################
# Module: script.py
# Author: Romain Lienard (rlienard@gmail.com)
# Description : A library providing several wrappers for IBM Security Secret Server REST APIs
# License : Apache Licence version 2.0 + Cookieware variant - if you like this module, please send me biscuits
###############################################################################################################

class Stub:
    def __init__(self,isss,payload,allowCreate=True):
        self.isss = isss
        self.payload = payload
        self.api = None
        self.allowCreate = allowCreate
    
    def __getitem__(self,item):
        for i in self.payload:
            if i.lower() == item.lower():
                return self.payload[i]
        
        if "items" in self.payload:
            for i in self.payload["items"]:
                if i["fieldName"].lower() == item.lower():
                    return i["itemValue"]
            
        print(("warning: item not found (%s)"%(item)))
        return None
        
    def __setitem__(self,item,value):
        for i in self.payload:
            if i.lower() == item.lower():
                self.payload[i] = value
                return
        
        if "items" in self.payload:
            for i in self.payload["items"]:
                if i["fieldName"].lower() == item.lower():
                    i["itemValue"] = value
                    return
        
        allitems = []
        if "items" in self.payload:
            for i in self.payload["items"]:
                allitems.append("+ " + i["fieldName"] + " : " + str(i["itemValue"]))
        
        for i in self.payload:
            if i != "items":
                allitems.append("- " + i + " : " + str(self.payload[i]))

        raise Exception ("The field '%s' is not available in this stub. Possible fields are : \n%s " % (item, "\n".join(allitems) ) )
        return
    
    def __str__(self):
        allitems = []
        if "items" in self.payload:
            for i in self.payload["items"]:
                allitems.append("+ " + i["fieldName"] + " : " + str(i["itemValue"]))
                
        for i in self.payload:
            if i != "items":
                allitems.append("- " + i + " : " + str(self.payload[i]))

        return "\n".join(allitems)
    
    def create(self):
        if not self.allowCreate:
            raise Exception("This secret has already been created. Please use stub.update() instead")
    
    def update(self):
        if self.allowCreate:
            raise Exception("This secret has not been created and cannot be updated. Please use stub.create() instead")
            
    def delete(self):
        if self.allowCreate:
            raise Exception("This secret has not been created yet and cannot be deleted.")
    
    
class ResultStub(Stub):
    def __init__(self,isss,payload):
        Stub.__init__(self,isss,payload)
    
class SecretStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.allowCreate = allowCreate
        if allowCreate:
            self.payload["siteId"] = 1
            
    def __repr__(self):
        return "<Secret id='%s' name='%s' template='%s' />" % (self["id"],self["name"],self["secretTemplateName"])
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createSecret(self)            
        self.allowCreate = False
        return self
            
    def update(self):
        Stub.update(self)
        self.payload = self.isss.putSecret(self)
        return self 
    
    def delete(self):
        Stub.delete(self)
        self.isss.deleteSecret(self)
        return None
    
    def getPermissions(self):
        return self.isss.getSecretPermission(self["id"])
    
    def generatePassword(self,fieldName="password"):
        for i in self.payload["items"]:
            if i["fieldName"].lower() == fieldName.lower():
                fieldId = i["fieldId"]
                rep = self.isss.post("/secret-templates/generate-password/" + str(fieldId))
                return rep
        raise "No '"+ fieldName +"' field in this template. Unable to create a password with appropriate policy."
    

class UserStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.payload["password"] = None
        self.payload["enabled"] = True
        self.allowCreate = allowCreate
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createUser(self)
        self.payload["password"] = None
        self.allowCreate = False
        return self
    
    def update(self):
        Stub.update(self)
        self.payload = self.isss.putUser(self)
        self.payload["password"] = None
        return self

    def delete(self):
        Stub.delete(self)
        self.isss.deleteUser(self)
        return None
    
    def __repr__(self):
        return "<User id='%s' username='%s' displayName='%s' />" % (self["id"],self["userName"],self["displayName"])
    
    
class GroupStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.allowCreate = allowCreate
        if allowCreate:
            self.payload["enabled"] = True
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createGroup(self)
        self.allowCreate = False
        return self

    def update(self):
        Stub.update(self)
        self.payload = self.isss.putGroup(self)
        return self
            
    def add(self,usernameOrUserId):
        
        if isinstance(usernameOrUserId,str):
            user = self.isss.getUser(usernameOrUserId)
            if len(user) ==0:
                raise Exception("Unable to find a user with the query '%s'"%(usernameOrUserId))
            elif len(user) >1:
                raise Exception("More than one user has been returned. Unable to add the user (consider using a userId instead)")
            else:
                user = user[0]
            self.isss.post("/groups/%s/users"%(self.payload["id"]),{"userId":user["id"]})
        else:
            self.isss.post("/groups/%s/users"%(self.payload["id"]),{"userId":usernameOrUserId})
        
        return self
    
    def remove(self,usernameOrUserId):
        
        if isinstance(usernameOrUserId,str):
            user = self.isss.getUser(usernameOrUserId)
            if len(user) ==0:
                raise Exception("Unable to find a user with the query '%s'"%(usernameOrUserId))
            elif len(user) >1:
                raise Exception("More than one user has been returned. Unable to remove the user (consider using a userId instead)")
            else:
                user = user[0]
            self.isss.delete("/groups/%s/users/%s" % (self.payload["id"],user["id"]) )
        else:
            self.isss.delete("/groups/%s/users/%s" % (self.payload["id"],usernameOrUserId) )
        
        return self
    
    def delete(self):
        Stub.delete(self)
        self.isss.deleteGroup(self)
        return None
    
    def getUsers(self):
        res = self.isss.get("/groups/%s/users"%(self.payload["id"]))
        users =  [{"id":u["userId"],"username":u["userName"]} for u in res["records"]]
        return users
    
    def __repr__(self):
        return "<Group id='%s' name='%s' enabled='%s' />" % (self["id"],self["name"],self["enabled"])
        
        
class SecretPermissionStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.allowCreate = allowCreate
        
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createSecretPermission(self)
        self.allowCreate = False
        return self  
    
    def update(self):
        Stub.update(self)
        self.payload = self.isss.putSecretPermission(self)
        return self 
    
    def delete(self):
        Stub.delete(self)
        self.isss.deleteSecretPermission(self)
        return None

    def __setitem__(self,item,value):
        if item.lower() == "secretaccessrolename":
            if value.lower() not in ["owner","view","list","edit"]:
                raise Exception("This specific stub value must be one of the following : Owner, View, List, Edit")
        Stub.__setitem__(self,item,value)
        
    def __repr__(self):
        return "<SecretPermission id='%s' secretAccessRoleName='%s' secretId='%s' username='%s' />" % (self["id"],self["secretAccessRoleName"],self["secretId"],self["userName"])
    
    
class FolderPermissionStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.allowCreate = allowCreate
        
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createFolderPermission(self)
        self.allowCreate = False
        return self  
    
    def update(self):
        Stub.update(self)
        self.payload = self.isss.putFolderPermission(self)
        return self 
    
    def delete(self):
        Stub.delete(self)
        self.isss.deleteFolderPermission(self)
        return None

    def __setitem__(self,item,value):
        if item.lower() == "secretaccessrolename":
            if value.lower() not in ["owner","view","list","edit"]:
                raise Exception("This specific stub value must be one of the following : Owner, View, List, Edit")
        Stub.__setitem__(self,item,value)
        
    def __repr__(self):
        return "<FolderPermission id='%s' secretAccessRoleName='%s' folderId='%s' username='%s' />" % (self["id"],self["secretAccessRoleName"],self["folderId"],self["userName"])
    
    
class FolderStub(Stub):
    def __init__(self,isss,payload,allowCreate=True):
        Stub.__init__(self,isss,payload)
        self.allowCreate = allowCreate
        
    
    def create(self):
        Stub.create(self)
        self.payload = self.isss.createFolder(self)
        self.allowCreate = False
        return self  
    
    def update(self):
        Stub.update(self)
        self.payload = self.isss.putFolder(self)
        return self 
    
    def getPermissions(self):
        return self.isss.getFolderPermission(self["id"])
    
    def delete(self):
        Stub.delete(self)
        self.isss.deleteFolder(self)
        return None
        
    def __repr__(self):
        return "<Folder id='%s' name='%s' path='%s'/>" % (self["id"],self["folderName"],self["folderPath"])


#!/usr/bin/env python3

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="isss",
    version="1.0.2",
    author="Romain Lienard",
    author_email="rlienard@fr.ibm.com",
    description="A unofficial library which provides wrappers around IBM Security Secret Server REST APIs. Also compatible with Thycotic Secret Server",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.ibm.com/marketplace/secret-server",
    packages=setuptools.find_packages(),
    scripts=["isss-fw","isss-license-formatter"],
    keywords=["isss","ibm", "thycotic", "secret", "server"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)
